
/*********************************************************************/
/*                                                                   */
/* Program name: ptol.c                                              */
/*                                                                   */
/* Description: Packed Decimal to long integer conversion            */
/*                                                                   */
/*  Statement:  Licensed Materials - Property of IBM                 */
/*                                                                   */
/*              CICS SupportPac CA0A                                 */
/*              (C) Copyright IBM Corp. 2000                         */
/*                                                                   */
/* input  - pointer to target area (long *)                          */
/*          pointer to source area (char *)                          */
/*                                                                   */
/*********************************************************************/

#include "pdecimal.h"

/* miscellaneous definitions */
#define POSITIVE      0x0c
#define UNSIGNED      0x0f
#define NEGATIVE      0x0d
#define MAX_POSITIVE  2147483647.0
#define MAX_NEGATIVE -2147483648.0

int ptol( long *pTarget, unsigned char *pSource )
{
  short  Rc = PDEC_OK;                       /* return code */
  short  SIndex;                             /* source index */
  long   Sign;                               /* sign multiplier */
  long   Power;                              /* power of 10 */
  double DTarget;                            /* work accumulator */

  /* Validate sign */
  if((( pSource[7] & 0x0f ) == POSITIVE ) ||
     (( pSource[7] & 0x0f ) == UNSIGNED ))
  {
    /* Positive value */
    Sign = +1;
  }
  else
  {
    if(( pSource[7] & 0x0f ) == NEGATIVE )
    {
      /* Negative value */
      Sign = -1;
    }
    else
    {
      /* Bad sign value */
      Rc = PDEC_BADDATA_PACKED;
    }
  }
 
  if( Rc == PDEC_OK )
  {
    /* Ok so far... now check rightmost digit */
    if(( pSource[7] & 0xf0 ) < 0xa0 )
    {
      DTarget = ( pSource[7] & 0xf0 ) >> 4;
    }
    else
    {
      /* Invalid number */
      Rc = PDEC_BADDATA_PACKED;
    }
  }

  if( Rc == PDEC_OK ) 
  {
    Power = 10;
    /* Main conversion loop */
    for( SIndex = 6; SIndex >= 0; SIndex -- )
    {
      if(( pSource[SIndex] & 0x0f ) < 0x0a )
      {
        /* Right-hand digit */
        DTarget += ( pSource[SIndex] & 0x0f ) * Power;
        Power *= 10;
        if(( pSource[SIndex] & 0xf0 ) < 0xa0 )
        {
          /* Left-hand digit */
          DTarget += (( pSource[SIndex] & 0xf0 ) >> 4 ) * Power;
          Power *= 10;
        }
        else
        {
          /* Invalid number */
          Rc = PDEC_BADDATA_PACKED;
          break;
        }
      }
      else
      {
        /* Invalid number */
        Rc = PDEC_BADDATA_PACKED;
        break;
      }
    }
  }

  if( Rc == PDEC_OK )
  {
    /* Conversion complete - now set the sign & check for overflow */
    DTarget *= Sign;
    if(( DTarget > MAX_POSITIVE ) || ( DTarget < MAX_NEGATIVE ))
    {
      Rc = PDEC_OVERFLOW;
    }
    *pTarget = DTarget;                      /* Return result */
  }

  return( Rc );
}
